<?php declare(strict_types=1);

namespace Convo\Proto;

use Psr\Http\Server\RequestHandlerInterface;

class AdminAuthRestHandler implements RequestHandlerInterface
{	
	/**
	 * @var \Convo\Core\Util\IHttpFactory
	 */
	private $_httpFactory;
	
	/**
	 * @var \Psr\Log\LoggerInterface
	 */
	private $_logger;
	
	/**
	 * @var \Convo\Proto\AdminUserDao
	 */
	private $_adminUserDao;
	
	public function __construct( \Psr\Log\LoggerInterface $logger, $adminUserDao, $httpFactory)
	{
		$this->_logger			= $logger;
		$this->_httpFactory		= $httpFactory;
		$this->_adminUserDao	= $adminUserDao;
	}
	
	public function handle(\Psr\Http\Message\ServerRequestInterface $request): \Psr\Http\Message\ResponseInterface
	{
		$info	=	new \Convo\Core\Rest\RequestInfo( $request);
		
		if ( $info->get() && $info->route( 'admin-auth/get-user'))
		{
			return $this->_performConvoProtoGetUserGet( $request, $info->getAuthUser());
		}
		
		if ( $info->post() && $info->route( 'admin-auth/login'))
		{
			return $this->_performConvoProtoLoginPost( $request);
		}
		
		if ( $info->post() && $info->route( 'admin-auth/logout'))
		{
			return $this->_performConvoProtoLogoutPost( $request);
		}
		
		throw new \Convo\Core\Rest\NotFoundException( 'Could not map ['.$info.']');
	}
	
	private function _performConvoProtoLogoutPost(\Psr\Http\Message\ServerRequestInterface $request)
	{
		unset( $_SESSION['username']);
		return $this->_httpFactory->buildResponse( array());
	}
	
	private function _performConvoProtoLoginPost(\Psr\Http\Message\ServerRequestInterface $request)
	{
		$data	=	$request->getParsedBody();
		
		if ( !isset( $data['username']) || empty( $data['username'])) {
			throw new \Convo\Core\Rest\InvalidRequestException( 'No authentication data in request');
		}
		
		if ( !isset( $data['password']) || empty( $data['password'])) {
			throw new \Convo\Core\Rest\InvalidRequestException( 'No authentication data in request');
		}
		
		try {
			$user	=	$this->_adminUserDao->findUser( $data['username']);
		} catch ( \Exception $e) {
			throw new \Convo\Core\Rest\NotAuthenticatedException( 'Failed to login', 0, $e);
		}
		
		if ( $user->getPassword() !== md5( $data['password'])) {
			throw new \Convo\Core\Rest\NotAuthenticatedException( 'Failed to login');
		}
		
		$_SESSION['username']	=	$user->getUsername();
		
		return $this->_performConvoProtoGetUserGet( $request, $user);
	}
	
	private function _performConvoProtoGetUserGet( \Psr\Http\Message\ServerRequestInterface $request, \Convo\Core\IAdminUser $user)
	{
		$data = $this->_adminUserDao->getPlatformConfig( $user->getId());

		return $this->_httpFactory->buildResponse( array(
			'user_id' => $user->getId(),
			'name' => $user->getName(),
		    'username' => $user->getUsername(),
			'email' => $user->getEmail(),
			'amazon_account_linked' => isset($data['amazon']['client_auth'])
		));
	}
	
	// UTIL
	public function __toString()
	{
		return get_class( $this).'[]';
	}
}