<?php declare(strict_types=1);

namespace Convo\Proto\Pckg\Elements;

class CategorySearch extends \Convo\Core\Workflow\AbstractWorkflowComponent implements \Convo\Core\Workflow\IConversationElement
{
    private $_json_path;
    private $_search;
    private $_result;
    
    public function __construct($properties) {
        
        parent::__construct( $properties);
        $this->_json_path = $properties['json_path'];
        $this->_search = $properties['search'];
        $this->_result = $properties['result'];
    }
    
    public function read( \Convo\Core\Workflow\IConvoRequest $request, \Convo\Core\Workflow\IConvoResponse $response)
    {
        $this->_logger->debug( 'Raw json path ['.$this->_json_path.']');
        $this->_logger->debug( 'Raw search ['.$this->_search.']');
        $this->_logger->debug( 'Raw result ['.$this->_result.']');
        
        $json_path	=   $this->evaluateString( $this->_json_path."categories.json");
        $search	=   $this->evaluateString( $this->_search);
        $result	=   $this->evaluateString( $this->_result);
        
        $this->_logger->debug( 'Using json path ['.$json_path.']');
        $this->_logger->debug( 'search ['.$search.']');
        $this->_logger->debug( 'var to store ['.$result.']');
        
        $json_file = file_get_contents($json_path);
        $json = json_decode($json_file, true);
        
        if (($json != $json_file) && $json)
        {
            $this->_logger->debug('JSON is valid');
        }
        else
        {
            $this->_logger->debug('JSON is invalid');
        }
        
        $params = $this->getService()->getServiceParams(\Convo\Core\Params\IServiceParamsScope::SCOPE_TYPE_SESSION);
        
        $num = 0;
        
        if ( empty( $search))
        {
            $num = count( $json);
            $this->_logger->debug('Number of categories ['.print_r( $num, true).']'); 
            
            $params->setServiceParam($result, [
                'count' => $num,
                'data' => $json
            ]);
        }
        
        if ( !empty( $search))
        {
            $length = count( $json);
            $found_categories = [];
            for ( $i = 0; $i < $length; $i++)
            {
                if ( stripos( $json[$i]['title'], $search) !== false) 
                {
                    $num++;
                    array_push($found_categories, $json[$i]);
                    
                    $params->setServiceParam($result, [
                        'count' => $num,
                        'data' => $found_categories
                    ]);
                }
            }
            
            $this->_logger->debug('Founde categories ['.print_r( $found_categories, true).']'); 
        }
        
    }
    
    
    // UTIL
    public function __toString()
    {
        return parent::__toString().'['.$this->_json_path.']';
    }
    
}

?>
