<?php declare(strict_types=1);

namespace Convo\Proto;

use Convo\Core\DataItemNotFoundException;

class ServiceUserDao
{
    /**
     * @var \Psr\Log\LoggerInterface
     */
    private $_logger;

    private $_basePath;

    public function __construct($logger, $basePath)
    {
        $this->_logger = $logger;

        $this->_basePath = $basePath;
    }

    public function getUsers()
    {
        return $this->_getUsers();
    }

    public function getUserById($userId)
    {
        $users = $this->_getUsers();

        foreach ($users as $user) {
            if ($user['userId'] === $userId) {
                return $user;
            }
        }

        throw new DataItemNotFoundException("Could not locate user with ID [$userId]");
    }

    public function getUserByEmail($email)
    {
        $users = $this->_getUsers();

        foreach ($users as $user) {
            if ($user['email'] === $email) {
                return $user;
            }
        }

        throw new DataItemNotFoundException("Could not locate user with email [$email]");
    }

    public function updateUser($userId, $data)
    {
        $users = $this->_getUsers();

        if (isset($data['userId'])) {
            unset($data['userId']);
        }
        
        $found = false;

        foreach ($users as &$user) {
            if ($found) {
                break;
            }

            if ($user['userId'] === $userId) {
                $user = array_replace_recursive($user, $data);
                
                $this->_logger->debug('Found and updated user ['.$userId.']['.print_r($user, true).']');
                $found = true;
            }
        }

        if ($found) {
            $this->_saveUsers($users);
        }
    }

    public function getUserByAuthCode($code, $type)
    {
        $users = $this->_getUsers();

        foreach ($users as $user)
        {
            if (isset($user['authCode'][$type])) {
                if ($user['authCode'][$type]['redeemed'] === true) {
                    throw new \Exception('Code has been redeemed.');
                }

                if ($user['authCode'][$type]['code'] === $code) {
                    return $user;
                }
            }
        }

        throw new DataItemNotFoundException('No user with this code of type ['.$type.'].');
    }

    public function getUserByRefreshToken($token, $type)
    {
        $users = $this->_getUsers();

        foreach ($users as $user)
        {
            if (isset($user['accessToken'][$type])) {
                if ($user['accessToken'][$type]['refresh_token'] === $token) {
                    return $user;
                }
            }
        }

        throw new DataItemNotFoundException('No user with this access token of type ['.$type.']');
    }

    public function getUserByAccessToken($token, $type)
    {
        $users = $this->_getUsers();

        foreach ($users as $user)
        {
            if (isset($user['accessToken'][$type])) {
                if ($user['accessToken'][$type]['access_token'] === $token) {
                    return $user;
                }
            }
        }

        throw new DataItemNotFoundException('No user with this access token of type ['.$type.']');
    }

    // UTIL
    public function __toString()
    {
        return get_class($this).'[]';
    }

    private function _saveUsers($users)
    {
        $path = $this->_basePath.'/users/__users-service.json';

        $contents = json_decode(file_get_contents($path), true);

        $contents['users'] = $users;

        file_put_contents($path, json_encode($contents));
    }

    private function _getUsers()
    {
        $path = $this->_basePath.'/users/__users-service.json';

        $contents = file_get_contents($path);

        if ($contents === false) {
            throw new \Exception('Could not open ['.$path.'] for reading.');
        }

        $json = json_decode($contents, true);

        return $json['users'];
    }
}